from typing import Optional, Union
from cnab.base.cnab_400 import CNAB400Registro2
from cnab.core.field import CNABField, CNABFieldType
from cnab.utils.dict_utils import set_if_has_value
from cnab.base.registro import Registro


class BradescoCnab400Registro2(CNAB400Registro2):
    _meta = {
        "tipo_registro": CNABField(
            length=1, default="2", validation=CNABFieldType.Int, required=True
        ),
        "mensagem_1": CNABField(
            length=80, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "mensagem_2": CNABField(
            length=80, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "mensagem_3": CNABField(
            length=80, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "mensagem_4": CNABField(
            length=80, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "data_desconto": CNABField(
            length=6, default="0", validation=CNABFieldType.Date, required=True
        ),
        "vlr_desconto": CNABField(
            length=11,
            default="0",
            validation=CNABFieldType.Decimal,
            precision=2,
            required=True,
        ),
        "data_desconto_2": CNABField(
            length=6, default="0", validation=CNABFieldType.Date, required=True
        ),
        "vlr_desconto_2": CNABField(
            length=11,
            default="0",
            validation=CNABFieldType.Decimal,
            precision=2,
            required=True,
        ),
        "filler": CNABField(
            length=7, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "carteira_banco": CNABField(
            length=3, default="0", validation=CNABFieldType.Int, required=True
        ),
        "agencia": CNABField(
            length=5, default="0", validation=CNABFieldType.Int, required=True
        ),
        "conta": CNABField(
            length=7, default="0", validation=CNABFieldType.Int, required=True
        ),
        "conta_dv": CNABField(
            length=1, default="0", validation=CNABFieldType.Int, required=True
        ),
        "nosso_numero": CNABField(
            length=11, default="0", validation=CNABFieldType.Int, required=True
        ),
        "nosso_numero_dv": CNABField(
            length=1, default="0", validation=CNABFieldType.Alfa, required=True
        ),
        "numero_registro": CNABField(
            length=6, default="0", validation=CNABFieldType.Int, required=True
        ),
    }



from cnab.base.registro import Registro
from cnab.base.registro_remessa import RegistroRemessa

class CNAB400Registro0(RegistroRemessa):
    def __init__(self, **kwargs: dict):
        self.counter = 1
        super().__init__(None, None, **kwargs)

    def append(self, child: Registro):
        super().append(child)
        self.counter += 1
    
    def inserir_lote(self, **kwargs: dict) -> "CNAB400Registro0":
        return self



from cnab.base.cnab_240 import CNAB240Registro9
from cnab.core.field import CNABField, CNABFieldType

class Santander240Registro9(CNAB240Registro9):
    _meta = {
        'codigo_banco': CNABField(
            length=3,
            default='033',
            validation=CNABFieldType.Int,
            required=True
        ),
        'codigo_lote': CNABField(
            length=4,
            default=9999,
            validation=CNABFieldType.Int,
            required=True
        ),
        'tipo_registro': CNABField(
            length=1,
            default='9',
            validation=CNABFieldType.Int,
            required=True
        ),
        'filler1': CNABField(
            length=9,
            default=' ',
            validation=CNABFieldType.Alfa,
            required=True
        ),
        'qtd_lotes': CNABField(
            length=6,
            default='1',
            validation=CNABFieldType.Int,
            required=True
        ),
        'qtd_registros': CNABField(
            length=6,
            default='0',
            validation=CNABFieldType.Int,
            required=True
        ),
        'filler2': CNABField(
            length=211,
            default=' ',
            validation=CNABFieldType.Alfa,
            required=True
        ),
    }



from typing import Optional, Union, TYPE_CHECKING
from cnab.base.remessa import Remessa
from cnab.utils.dict_utils import set_if_has_value
from .registro0 import ItauCnab400Registro0
from .registro9 import ItauCnab400Registro9

if TYPE_CHECKING:
    from .registro0 import ItauCnab400Registro0
#
class CNAB400Itau(Remessa):
    header: ItauCnab400Registro0
    registro0_class = ItauCnab400Registro0
    registro9_class = ItauCnab400Registro9

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def inserir_lote(self, **kwargs) -> "ItauCnab400Registro0":
        return super().inserir_lote(**kwargs)



from .registro0 import CNAB240Registro0
from .registro1 import CNAB240Registro1
from .registro3 import CNAB240Registro3
from .registro5 import CNAB240Registro5
from .registro9 import CNAB240Registro9

__all__ = [
    'CNAB240Registro0',
    'CNAB240Registro1',
    'CNAB240Registro3',
    'CNAB240Registro5',
    'CNAB240Registro9',
]



from cnab.base.cnab_240 import CNAB240Registro0
from cnab.core.field import (
    CNABField,
    CNABFieldType,
    CNABCreatedDateField,
)
from .registro1 import Caixa240Registro1


class Caixa240Registro0(CNAB240Registro0):
    registro1_class = Caixa240Registro1
    _meta = {
        "codigo_banco": CNABField(  # 01.0
            length=3,
            default="104",
            validation=CNABFieldType.Int,
            required=True
        ),
        "codigo_lote": CNABField(  # 02.0
            length=4, default="0000", validation=CNABFieldType.Int, required=True
        ),
        "tipo_registro": CNABField(  # 03.0
            length=1, default="0", validation=CNABFieldType.Int, required=True
        ),
        "filler1": CNABField(  #  04.0
            length=9, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "tipo_inscricao": CNABField(  # 05.0
            length=1, default="", validation=CNABFieldType.Int, required=True
        ),
        "numero_inscricao": CNABField(  # 06.0
            length=14, default="", validation=CNABFieldType.Int, required=True
        ),
        "filler2": CNABField(  # 07.0
            length=20, default="0", validation=CNABFieldType.Int, required=True
        ),
        "agencia": CNABField(  # 08.0
            length=5, default="", validation=CNABFieldType.Int, required=True
        ),
        "agencia_dv": CNABField(  # 09.0
            length=1, default="", validation=CNABFieldType.Int, required=True
        ),
        "codigo_beneficiario": CNABField( # 10.0
            # TODO: lidar com diferentes layouts
            length=7, default="", validation=CNABFieldType.Int, required=True 
        ),
        "filler3": CNABField(  # 11.0/12.0
            length=7, default="0", validation=CNABFieldType.Int, required=True
        ),
        "nome_empresa": CNABField(  # 13.0
            length=30, default="", validation=CNABFieldType.Alfa, required=True
        ),
        "nome_banco": CNABField(  # 14.0
            length=30, default="CAIXA", validation=CNABFieldType.Alfa, required=True
        ),
        "filler4": CNABField(  # 15.0
            length=10, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "codigo_remessa": CNABField(  # 16.0
            length=1, default="1", validation=CNABFieldType.Int, required=True
        ),
        "data_geracao": CNABCreatedDateField(  # 17.0
            length=8,
            validation=CNABFieldType.Date,
            required=True,
        ),
        "hora_geracao": CNABCreatedDateField(  # 18.00
            length=6,
            validation=CNABFieldType.Time,
            required=True,
        ),
        "numero_sequencial_arquivo": CNABField(  # 19.0
            length=6, default="", validation=CNABFieldType.Int, required=True
        ),
        "versao_layout": CNABField(  # 20.0
            # TODO: lidar com diferentes layouts
            length=3, default="101", validation=CNABFieldType.Int, required=True
        ),
        "densidade_gravacao": CNABField(  # 21.0
            length=5, default="0", validation=CNABFieldType.Int, required=True
        ),
        "filler5": CNABField(  # 22.0
            length=20, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "reservado_empresa": CNABField(  # 23.0
            length=20, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "filler6": CNABField(  # 24.0
            length=4, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "filler7": CNABField(  # 25.0
            length=25, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
    }

    def get_versao_layout(self):
        if self._data.get('versao_layout'):
            return self._data.get('versao_layout')

        codigo_beneficiario = self._data.get('codigo_beneficiario')
        if len(codigo_beneficiario) > 6:
            return '107'
        return '101'
    
    def get_codigo_beneficiario(self):
        codigo_beneficiario = self._data.get('codigo_beneficiario')
        versao_layout = self.get_versao_layout()
        if versao_layout == '101':
            code = str(codigo_beneficiario).rjust(6, '0')
            return f"{code}0"
        if versao_layout == '107':
            return str(codigo_beneficiario).rjust(7, '0')
        # TODO: raise exception here



from unidecode import unidecode

def prepare_text(text: str) -> str:
    return unidecode(text.strip()).upper()

def format_alfa(value, field):
    value = prepare_text(str(value))
    if len(value) > field.length:
        value = value[0:field.length]
    return value.ljust(field.length, ' ')



from cnab.base.cnab_240 import CNAB240Registro3
from cnab.core.field import CNABField, CNABFieldType


class BancoBrasil240Registro3Q(CNAB240Registro3):
    _meta = {
        "codigo_banco": CNABField(  # 1.3Q
            length=3, default="001", validation=CNABFieldType.Int, required=True
        ),
        "codigo_lote": CNABField(  # 2.3Q
            length=4, default=1, validation=CNABFieldType.Int, required=True
        ),
        "tipo_registro": CNABField(  # 3.3Q
            length=1, default="3", validation=CNABFieldType.Int, required=True
        ),
        "numero_registro": CNABField(  # 4.3Q
            length=5, default="2", validation=CNABFieldType.Int, required=True
        ),
        "seguimento": CNABField(  # 5.3Q
            length=1, default="Q", validation=CNABFieldType.Alfa, required=True
        ),
        "filler1": CNABField(  # 6.3Q
            length=1, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "codigo_movimento": CNABField(  # 7.3Q
            length=2,
            default="01",  # entrada de titulo
            validation=CNABFieldType.Int,
            required=True,
        ),
        # - ------------------ até aqui é igual para todo registro tipo 3
        "tipo_inscricao": CNABField(  # 8.3Q
            length=1, default="0", validation=CNABFieldType.Int, required=True
        ),
        "numero_inscricao": CNABField(  # 9.3Q
            length=15, default="0", validation=CNABFieldType.Int, required=True
        ),
        "nome_pagador": CNABField(  # 10.3Q
            length=40, default="", validation=CNABFieldType.Alfa, required=True
        ),
        "endereco_pagador": CNABField(  # 11.3Q
            length=40, default="", validation=CNABFieldType.Alfa, required=True
        ),
        "bairro_pagador": CNABField(  # 12.3Q
            length=15, default="", validation=CNABFieldType.Alfa, required=True
        ),
        "cep_pagador": CNABField(  # 13.3Q
            length=5, default="0", validation=CNABFieldType.Int, required=True
        ),
        "cep_sufixo": CNABField(  # 14.3Q
            length=3, default="0", validation=CNABFieldType.Int, required=True
        ),
        "cidade_pagador": CNABField(  # 15.3Q
            length=15, default="", validation=CNABFieldType.Alfa, required=True
        ),
        "uf_pagador": CNABField(  # 16.3Q
            length=2,
            default="",  # combrança com registro
            validation=CNABFieldType.Alfa,
            required=True,
        ),
        "tipo_incricao_avalista": CNABField(  # 17.3Q
            length=1, default="0", validation=CNABFieldType.Int, required=True
        ),
        "numero_incricao_avalista": CNABField(  # 18.3
            length=15, default="0", validation=CNABFieldType.Int, required=True
        ),
        "nome_avalista": CNABField(  # 18.3Q
            length=40, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "codigo_banco_correspondente": CNABField(  # 18.3Q
            length=3, default="0", validation=CNABFieldType.Int, required=True
        ),
        "nosso_numero_banco_correspondente": CNABField(  # 19.3Q   Campo de preenchimento obrigatório; preencher com Seu Número de controle do título
            length=20,
            default=" ",  # este espaço foi colocado para passa a validação para os seters do generico
            validation=CNABFieldType.Alfa,
            required=True,
        ),
        "filler4": CNABField(  # 19.3Q
            length=8, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
    }



from cnab.core.exceptions import IsNotValidDateTimeError

def validate_date(value, field):
    if value == '0':
        return value
    if not hasattr(value, 'strftime'):
        raise IsNotValidDateTimeError(field.name)
    return value



from .required import validate_required
from .integer import validate_integer
from .decimal import validate_decimal
from .date import validate_date

__all__ = [
    'validate_required', 'validate_integer', 'validate_decimal', 'validate_date'
]



from cnab.base.remessa import Remessa
from .registro0 import Caixa240Registro0
from .registro9 import Caixa240Registro9

class CNAB240Caixa(Remessa):
    header: Caixa240Registro0
    registro0_class = Caixa240Registro0
    registro9_class = Caixa240Registro9




from abc import ABC, abstractmethod
from typing import List, Optional, Callable
from cnab.core.field import CNABField
from cnab.core import exceptions

class Registro(ABC):
    parent: Optional["Registro"] = None
    header: Optional["Registro"] = None

    _children: List["Registro"] = []
    _meta: Optional[dict] = None
    _data: Optional[dict] = None

    def __init__(self, header: Optional["Registro"], parent: Optional["Registro"], **kwargs: dict):
        self.header = header
        self._children = []
        self.parent = parent
        self._data = kwargs
        
        if not parent:
            return
        parent.append(self)

    def get_field(self, key: str) -> CNABField:
        item = self._meta[key]
        if not isinstance(item, CNABField):
            raise exceptions.IsNotCNABFieldError(key)
        item.name = key
        return item

    def get_unformated(self, key, default=None):
        value = None
        if hasattr(self, f'get_{key}'):
            fn = getattr(self, f'get_{key}')
            value = fn()

        if not value:
            value = self._data.get(key, default)
        return value
    
    def get_default(self, field: CNABField):
        if not field.default:
            return field.default
        
        if isinstance(field.default, Callable):
            return field.default()
        
        return field.default
    
    def get_value(self, key, default=None):
        value = self.get_unformated(key, default)
        field = self.get_field(key)
        return field.format_value(value)
    
    def get_data_or_parent(self, field: str):
        if self._data.get(field):
            return self._data.get(field)
        if not self.header:
            return None
        return self.header._data.get(field)

    def get_codigo_carteira(self):
        value = self._data.get('codigo_carteira')
        if not value:
            return None
        return int(value)

    def append(self, child: "Registro"):
        self._children.append(child)

    @abstractmethod
    def get_text(self) -> str:
        raise NotImplementedError("get_text() is not implemented")



from datetime import datetime
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from cnab.core.field import CNABField

def format_date(value: datetime, field: "CNABField"):
    if value == '0':
        return value * field.length
    if field.length == 8:
        return value.strftime("%d%m%Y")
    if field.length == 6:
        return value.strftime("%d%m%y")
    # TODO: raise custom exception here
    raise Exception("invalid field")



from cnab.base.cnab_240 import CNAB240Registro5
from cnab.core.field import CNABField, CNABFieldType


class Caixa240Registro5(CNAB240Registro5):
    _meta = {
        'codigo_banco': CNABField(      #01.5
			length=3,
			default='104',
			validation=CNABFieldType.Int,
			required=True),
		'codigo_lote': CNABField(       #02.5
			length=4,
			default=1,
			validation=CNABFieldType.Int,
			required=True),
		'tipo_registro': CNABField(     #03.5
			length=1,
			default='5',
			validation=CNABFieldType.Int,
			required=True),
		'filler1': CNABField(          #04.5
			length=9,
			default=' ',
			validation=CNABFieldType.Alfa,
			required=True),
		'qtd_registros': CNABField(      #05.5
			length=6,
			default=' ',
			validation=CNABFieldType.Int,
			required=True),
        'qtd_registros_cobranca_simples': CNABField(      #06.5
			length=6,
			default='0',
			validation=CNABFieldType.Int,
			required=True),
		'somatorio_valores_simples': CNABField(# 07.5
			length=15,
			default='0',
			validation=CNABFieldType.Decimal,
			precision=2,
			required=True),
        'qtd_registros_cobranca_caucionadas': CNABField(      # 08.5
			length=6,
			default='0',
			validation=CNABFieldType.Int,
			required=True),
        'somatorio_valores_caucionadas': CNABField( # 09.5
			length=15,
			default='0',
			validation=CNABFieldType.Decimal,
			precision=2,
			required=True),
        'qtd_registros_cobranca_descontada': CNABField( # 10.5
			length=6,
			default='0',
			validation=CNABFieldType.Int,
			required=True),
        'somatorio_valores_descontada': CNABField( # 11.5
			length=15,
			default='0',
			validation=CNABFieldType.Decimal,
			precision=2,
			required=True),	
		'filler2': CNABField( # 12.5
			length=31,
			default=' ',
			validation=CNABFieldType.Alfa,
			required=True),
		'filler3': CNABField( # 13.5
			length=117,
			default=' ',
			validation=CNABFieldType.Alfa,
			required=True),
    }



from cnab.base.cnab_400 import CNAB400Registro2
from cnab.core.field import CNABField, CNABFieldType


class ItauCnab400Registro2(CNAB400Registro2):
    _meta = {
        "tipo_registro": CNABField(
            length=1, default="2", validation=CNABFieldType.Int, required=True
        ),
        "codigo_multa": CNABField(  # 24.3P
            length=1, default="1", validation=CNABFieldType.Alfa, required=True
        ),
        "data_multa": CNABField(  # 31.3P
            length=8, default="0", validation=CNABFieldType.Date, required=True
        ),
        "vlr_multa": CNABField(  # 29.3P
            length=11,
            default="0",
            validation=CNABFieldType.Decimal,
            precision=2,
            required=True,
        ),
        "filler2": CNABField(  # 32.3P
            length=371, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "numero_registro": CNABField(  # 4.3R
            length=6, default="0", validation=CNABFieldType.Int, required=True
        ),
    }



from cnab.base.cnab_240 import CNAB240Registro3
from cnab.core.field import CNABField, CNABFieldType


class Caixa240Registro3S3(CNAB240Registro3):
    _meta = {
        "codigo_banco": CNABField(  # 1.3S
            length=3, default="104", validation=CNABFieldType.Int, required=True
        ),
        "codigo_lote": CNABField(  # 2.3S
            length=4, default=1, validation=CNABFieldType.Int, required=True
        ),
        "tipo_registro": CNABField(  # 3.3S
            length=1, default="3", validation=CNABFieldType.Int, required=True
        ),
        "numero_registro": CNABField(  # 4.3S
            length=5, default="0", validation=CNABFieldType.Int, required=True
        ),
        "seguimento": CNABField(  # 5.3S
            length=1, default="S", validation=CNABFieldType.Alfa, required=True
        ),
        "filler1": CNABField(  # 6.3S
            length=1, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "codigo_movimento": CNABField(  # 7.3S
            length=2,
            default="01",  # entrada de titulo
            validation=CNABFieldType.Int,
            required=True,
        ),
        # - ------------------ até aqui é igual para todo registro tipo 3
        "tipo_impressao": CNABField(  # 8.3S
            length=1, default="0", validation=CNABFieldType.Int, required=True
        ),
        "mensagem_5": CNABField(  # 9.3S
            length=40, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "mensagem_6": CNABField(  # 10.3S
            length=40, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "mensagem_7": CNABField(  # 11.3S
            length=40, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "mensagem_8": CNABField(  # 12.3S
            length=40, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "filler2": CNABField(  # 13.3S
            length=40, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "filler3": CNABField(  # 14.3S
            length=22, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
    }



from typing import Optional
from cnab.base.cnab_240 import CNAB240Registro3
from cnab.core.field import CNABField, CNABFieldType
from cnab.base.registro import Registro
from cnab.utils.check_digit import compute_check_digit
from .registro3Q import Santander240Registro3Q
from .registro1 import Santander240Registro1
from .registro3R import Santander240Registro3R

class Santander240Registro3P(CNAB240Registro3):
    _meta = {
        "codigo_banco": CNABField(
            length=3, default="033", validation=CNABFieldType.Int, required=True
        ),
        "codigo_lote": CNABField(
            length=4, default=1, validation=CNABFieldType.Int, required=True
        ),
        "tipo_registro": CNABField(
            length=1, default="3", validation=CNABFieldType.Int, required=True
        ),
        "numero_registro": CNABField(
            length=5, default="0", validation=CNABFieldType.Int, required=True
        ),
        "seguimento": CNABField(
            length=1, default="P", validation=CNABFieldType.Alfa, required=True
        ),
        "filler1": CNABField(
            length=1, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "codigo_movimento": CNABField(
            length=2,
            default="01",  # entrada de titulo
            validation=CNABFieldType.Int,
            required=True,
        ),
        # - ------------------ até aqui é igual para todo registro tipo 3
        "agencia": CNABField(
            length=4, default="", validation=CNABFieldType.Int, required=True
        ),
        "agencia_dv": CNABField(
            length=1, default="", validation=CNABFieldType.Int, required=True
        ),
        "conta": CNABField(
            length=9, default="0", validation=CNABFieldType.Int, required=True
        ),
        "conta_dv": CNABField(
            length=1, default="", validation=CNABFieldType.Int, required=True
        ),
        "codigo_beneficiario": CNABField(
            length=9, default="", validation=CNABFieldType.Int, required=True
        ),
        "codigo_beneficiario_dv": CNABField(
            length=1, default="", validation=CNABFieldType.Int, required=True
        ),
        "filler2": CNABField(
            length=2, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "nosso_numero": CNABField(
            length=13, default="", validation=CNABFieldType.Int, required=True
        ),
        "tipo_cobranca": CNABField(
            length=1, default="5", validation=CNABFieldType.Int, required=True
        ),
        "forma_cadastramento": CNABField(
            length=1,
            default="1",  # '1' = Cobrança Registrada (Rápida e Eletrônica com Registro)
            validation=CNABFieldType.Int,
            required=True,
        ),
        "tipo_documento": CNABField(
            length=1,
            default="1",  # 1- Tradicional , 2- Escritural
            validation=CNABFieldType.Int,
            required=True,
        ),
        "filler3": CNABField(
            length=1, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "filler4": CNABField(
            length=1, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "seu_numero": CNABField(
            length=15, default="", validation=CNABFieldType.Int, required=True
        ),
        "data_vencimento": CNABField(
            length=8, default="", validation=CNABFieldType.Date, required=True
        ),
        "valor": CNABField(
            length=13,
            default="",
            validation=CNABFieldType.Decimal,
            precision=2,
            required=True,
        ),
        "agencia_cobradora": CNABField(
            length=4, default="0", validation=CNABFieldType.Int, required=True
        ),
        "agencia_cobradora_dv": CNABField(
            length=1, default="0", validation=CNABFieldType.Int, required=True
        ),
        "filler5": CNABField(
            length=1, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "especie_titulo": CNABField(
            length=2, default="2", validation=CNABFieldType.Int, required=True
        ),
        "aceite": CNABField(
            length=1, default="N", validation=CNABFieldType.Alfa, required=True
        ),
        "data_emissao": CNABField(
            length=8, default="", validation=CNABFieldType.Date, required=True
        ),
        #
        # Códigos dos juros de mora
        # 1 = Valor por dia - Informar no campo o valor/dia a mora a ser cobrada.
        # 2 = Taxa Mensal - Informar no campo taxa mensal o percentual a ser aplicado sobre valor do titulo que será calculado por dia de atraso.
        # 3 = Isento
        # 4 = Utilizar comissão permanência do Banco por dia de atraso
        # 5 = Tolerância valor por dia (cobrar juros a partir de)
        # 6 = Tolerância taxa mensal (cobrar juros a partir de)
        # Para o código igual 4, o campo “taxa mensal” não deverá conter informação.
        #
        "codigo_juros": CNABField(
            length=1, default="0", validation=CNABFieldType.Int, required=True
        ),
        "data_juros": CNABField(
            length=8, default="0", validation=CNABFieldType.Date, required=True
        ),
        "vlr_juros": CNABField(
            length=13,
            default="0",
            validation=CNABFieldType.Decimal,
            precision=2,
            required=True,
        ),
        #
        # 0 = ISENTO
        # 1 = Valor fixo ate a data informada – Informar o valor no campo “valor de desconto a ser concedido”.
        # 2 = Percentual ate a data informada – Informar o percentual no campo “percentual de desconto a ser concedido”
        # 3 = Valor por antecipação por dia corrido - Informar o valor no campo “valor de desconto a ser concedido”
        # 4 = Valor por antecipação dia útil - Informar o valor no campo “valor de desconto a ser concedido” Para os código 1 e 2 será obrigatório a informação da “data” NOTA: é possível informar até duas ocorrências de desconto, por ex.:
        #      Segmento P : Valor do titulo R$ 100,00 Vencimento 30/09/1998
        #      ( Desconto 1 R$ 10,00 p/ pagamento até 25/09/1998
        #      Segmento R: < Desconto 2 R$ 8,00 p/ pagamento até 20/09/1998
        #
        "codigo_desconto": CNABField(
            length=1, default="0", validation=CNABFieldType.Int, required=True
        ),
        "data_desconto": CNABField(
            length=8, default="0", validation=CNABFieldType.Date, required=True
        ),
        "vlr_desconto": CNABField(
            length=13,
            default="0",
            validation=CNABFieldType.Decimal,
            precision=2,
            required=True,
        ),
        "vlr_IOF": CNABField(
            length=13,
            default="0",
            validation=CNABFieldType.Decimal,
            precision=2,
            required=True,
        ),
        "vlr_abatimento": CNABField(
            length=13,
            default="0",
            validation=CNABFieldType.Decimal,
            precision=2,
            required=True,
        ),
        "seu_numero2": CNABField(
            length=25, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        #
        # 0 NAO PROTESTAR
        # 1 PROTESTAR DIAS CORRIDOS
        # 2 PROTESTAR DIAS UTEIS
        # 3 UTILIZAR PERFIL BENEFICIÁRIO
        # 9 CANCELAMENTO DE PROTESTO AUTOMATICO
        #
        "protestar": CNABField(
            length=1, default=3, validation=CNABFieldType.Alfa, required=True
        ),
        "prazo_protesto": CNABField(
            length=2, default="0", validation=CNABFieldType.Int, required=True
        ),
        #
        # 1 BAIXAR / DEVOLVER
        # 2 NAO BAIXAR / NAO DEVOLVER
        # 3 UTILIZAR PERFIL BENEFICIÁRIO
        #
        "baixar": CNABField(
            length=1, default="1", validation=CNABFieldType.Int, required=True
        ),
        "filler6": CNABField(
            length=1, default="0", validation=CNABFieldType.Int, required=True
        ),
        "prazo_baixar": CNABField(
            length=2, default="90", validation=CNABFieldType.Int, required=True
        ),
        "codigo_moeda": CNABField(
            length=2, default="00", validation=CNABFieldType.Int, required=True
        ),
        "filler7": CNABField(
            length=11, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
    }

    def _inserir_detalhe(self, **kwargs: dict):
        if int(kwargs.get('codigo_movimento')) == 2:
            return
        
        Santander240Registro3Q(self.header, self, self.lote, **kwargs)
        
        desconto2 = kwargs.get('codigo_desconto2')
        desconto3 = kwargs.get('codigo_desconto3')
        vlr_multa = kwargs.get('vlr_multa')
        informacao_pagador = kwargs.get('informacao_pagador')
        if not desconto2 and not desconto3 and not vlr_multa and not informacao_pagador:
            return
        
        Santander240Registro3R(self.header, self, self.lote, **kwargs)

    def __init__(
        self,
        header: Optional["Registro"],
        parent: Optional["Registro"],
        lote: Santander240Registro1,
        **kwargs: dict
    ):
        super().__init__(header, parent, lote, **kwargs)

        self._inserir_detalhe(**kwargs)

    def get_codigo_beneficiario(self):
        return self.get_data_or_parent('conta')
    
    def get_codigo_beneficiario_dv(self):
        return self.get_data_or_parent('conta_dv')
    
    def get_nosso_numero(self):
        num = str(self._data.get('nosso_numero'))
        if num == "0" or num == " ":
            return num
        return num + str(compute_check_digit(num))




from cnab.base.cnab_240 import CNAB240Registro1
from cnab.core.field import CNABCreatedDateField, CNABField, CNABFieldType
from .registro5 import Caixa240Registro5

class Caixa240Registro1(CNAB240Registro1):
    registro5_class = Caixa240Registro5
    _meta = {
        'codigo_banco': CNABField(#01.1
			length=3,
			default='104',
			validation=CNABFieldType.Int,
			required=True),
		'codigo_lote': CNABField(#02.1
			length=4,
			default=1,
			validation=CNABFieldType.Int,
			required=True),
		'tipo_registro': CNABField(#3.1
			length=1,
			default=1,
			validation=CNABFieldType.Int,
			required=True),
		'operacao': CNABField(#04.1
			length=1,
			default='R',
			validation=CNABFieldType.Alfa,
			required=True),
		'tipo_servico': CNABField(#05.1
			length=2,
			default='01',
			validation=CNABFieldType.Int,
			required=True),
		'filler1': CNABField(#06.1
			length=2,
			default='0',
			validation=CNABFieldType.Int,
			required=True),
		'versao_layout': CNABField(#07.1
            # TODO: lidar com diferentes layouts
			length=3,
			default='060',
			validation=CNABFieldType.Int,
			required=True),
		'filler2': CNABField(#08.1
			length=1,
			default=' ',
			validation=CNABFieldType.Alfa,
			required=True),
		'tipo_inscricao': CNABField(#09.1
			length=1,
			default='',
			validation=CNABFieldType.Int,
			required=True),
		'numero_inscricao': CNABField(#10.1
			length=15,
			default='',
			validation=CNABFieldType.Int,
			required=True),
        "codigo_beneficiario": CNABField( # 11.1
            # TODO: lidar com diferentes layouts
            length=7, default="", validation=CNABFieldType.Int, required=True 
        ),
        'filler3': CNABField( # 11.1A
			length=13,
			default='0',
			validation=CNABFieldType.Int,
			required=True),
		'agencia': CNABField(#12.1
			length=5,
			default='',
			validation=CNABFieldType.Int,
			required=True),
		'agencia_dv': CNABField(#13.1
			length=1,
			default='',
			validation=CNABFieldType.Int,
			required=True),
        "codigo_beneficiario_6": CNABField( # 14.1
            # TODO: lidar com diferentes layouts
            length=6, default="0", validation=CNABFieldType.Int, required=True 
        ),
        "codigo_modelo_personalizado": CNABField(
            length=7, default="0", validation=CNABFieldType.Int, required=True,
		),
		'filler4': CNABField(#16.1
			length = 1,
			default = '0',
			validation = CNABFieldType.Int,
			required = True),
		'nome_empresa': CNABField( #17.1
			length=30,
			default='',
			validation=CNABFieldType.Alfa,
			required=True),
		'mensagem_fixa1': CNABField( # 18.1
            # mensagems 1 e 2 : somente use para mensagens que serao impressas de forma identica em todos os boletos do lote
			length=40,
			default=' ',
			validation=CNABFieldType.Alfa,
			required=True),
		'mensagem_fixa2': CNABField( # 19.1
            # mensagems 1 e 2 : somente use para mensagens que serao impressas de forma identica em todos os boletos do lote
			length=40,
			default=' ',
			validation=CNABFieldType.Alfa,
			required=True),
        "numero_remessa": CNABField(  # 20.1
            length=8, default="0", validation=CNABFieldType.Int, required=True
        ),
        "data_gravacao": CNABCreatedDateField(  # 21.1
            length=8,
            validation=CNABFieldType.Date,
            required=True,
        ),
		'filler5' : CNABField( # 22.1
			length = 8,
			default = '0',
			validation = CNABFieldType.Int,
			required = True),
		'filler6' : CNABField( # 23.1
			length = 33,
			default = ' ',
			validation = CNABFieldType.Alfa,
			required = True),
    }
    
    def inserir_detalhe(self, **kwargs):
        from .registro3P import Caixa240Registro3P
        return Caixa240Registro3P(self.header, self, self, **kwargs)

    def get_versao_layout(self):
        if self._data.get('versao_layout'):
            return self._data.get('versao_layout')

        codigo_beneficiario = self.get_data_or_parent('codigo_beneficiario')
        if len(codigo_beneficiario) > 6:
            return '067'
        return '060'
    
    def get_codigo_beneficiario_6(self):
        if self._data.get('codigo_beneficiario_6'):
            return self._data.get('codigo_beneficiario_6')
        codigo_beneficiario = self.get_data_or_parent('codigo_beneficiario')
        if len(codigo_beneficiario) == 6:
            return codigo_beneficiario
        return "000000"

    def get_codigo_beneficiario(self):
        codigo_beneficiario = self.get_data_or_parent('codigo_beneficiario')
        versao_layout = self.get_versao_layout()
        if versao_layout == '060':
            code = str(codigo_beneficiario).rjust(6, '0')
            return f"{code}0"
        if versao_layout == '067':
            return str(codigo_beneficiario).rjust(7, '0')
		# TODO: raise exception here



from typing import Optional, Union, TYPE_CHECKING
from cnab.base.cnab_240 import CNAB240Registro1
from cnab.core.field import CNABField, CNABCreatedDateField, CNABFieldType
from cnab.utils.dict_utils import set_if_has_value
from .registro5 import BancoBrasil240Registro5

if TYPE_CHECKING:
    from .registro0 import BancoBrasil240Registro0

class BancoBrasil240Registro1(CNAB240Registro1):
    registro5_class = BancoBrasil240Registro5
    _meta = {
        "codigo_banco": CNABField(  # 01.1
            length=3, default="001", validation=CNABFieldType.Int, required=True
        ),
        "codigo_lote": CNABField(  # 02.1
            length=4, default=1, validation=CNABFieldType.Int, required=True
        ),
        "tipo_registro": CNABField(  # 3.1
            length=1, default=1, validation=CNABFieldType.Int, required=True
        ),
        "operacao": CNABField(  # 04.1
            length=1, default="R", validation=CNABFieldType.Alfa, required=True
        ),
        "tipo_servico": CNABField(  # 05.1
            length=2, default="01", validation=CNABFieldType.Int, required=True
        ),
        "filler1": CNABField(  # 06.1
            length=2, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "versa_layout": CNABField(  # 07.1
            length=3, default="000", validation=CNABFieldType.Int, required=True
        ),
        "filler2": CNABField(  # 08.1
            length=1, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "tipo_inscricao": CNABField(  # 09.1
            length=1, default="0", validation=CNABFieldType.Int, required=True
        ),
        "numero_inscricao": CNABField(  # 10.1
            length=15, default="", validation=CNABFieldType.Int, required=True
        ),
        "convenio": CNABField(  # 11.1
            length=9, default="0", validation=CNABFieldType.Int, required=True
        ),
        "cobranca": CNABField(  # 11.1
            length=4, default="0014", validation=CNABFieldType.Int, required=True
        ),
        "carteira": CNABField(  # 11.1
            length=2, default="0", validation=CNABFieldType.Int, required=True
        ),
        "variacao": CNABField(  # 11.1
            length=3, default="000", validation=CNABFieldType.Int, required=True
        ),
        "situacao_arquivo": CNABField(  # 11.1
            length=2, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "agencia": CNABField(  # 12.1
            length=5, default="", validation=CNABFieldType.Int, required=True
        ),
        "agencia_dv": CNABField(  # 13.1
            length=1, default="", validation=CNABFieldType.Alfa, required=True
        ),
        "conta": CNABField(  # 14.1
            length=12, default="", validation=CNABFieldType.Int, required=True
        ),
        "conta_dv": CNABField(  # 15.1
            length=1, default="", validation=CNABFieldType.Alfa, required=True
        ),
        "filler3": CNABField(  # 16.1
            length=1, default="0", validation=CNABFieldType.Alfa, required=True
        ),
        "nome_empresa": CNABField(  # 17.1
            length=30, default="", validation=CNABFieldType.Alfa, required=True
        ),
        "mensagem_fixa1": CNABField(  # 18.1 mensagems 1 e 2 : somente use para mensagens que serao impressas de forma identica em todos os boletos do lote
            length=40, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "mensagem_fixa2": CNABField(  # 19.1 mensagems 1 e 2 : somente use para mensagens que serao impressas de forma identica em todos os boletos do lote
            length=40, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "numero_remessa": CNABField(  # 20.1
            length=8, default="0", validation=CNABFieldType.Int, required=True
        ),
        "data_gravacao": CNABCreatedDateField(  # 21.1
            length=8,
            validation=CNABFieldType.Date,
            required=True,
        ),
        "filler4": CNABField(  # 22.1
            length=8, default="0", validation=CNABFieldType.Int, required=True
        ),
        "filler5": CNABField(  # 23.1
            length=33, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
    }

    def __init__(
        self,
        header: "BancoBrasil240Registro0",
        parent: "BancoBrasil240Registro0",
        *,
        codigo_banco: Optional[Union[str, int]] = None,
        codigo_lote: Optional[Union[str, int]] = None,
        tipo_registro: Optional[Union[str, int]] = None,
        operacao: Optional[Union[str, int]] = None,
        tipo_servico: Optional[Union[str, int]] = None,
        tipo_inscricao: Optional[Union[str, int]] = None,
        numero_inscricao: Optional[Union[str, int]] = None,
        convenio: Optional[Union[str, int]] = None,
        cobranca: Optional[Union[str, int]] = None,
        carteira: Optional[Union[str, int]] = None,
        variacao: Optional[Union[str, int]] = None,
        agencia: Optional[Union[str, int]] = None,
        agencia_dv: Optional[Union[str, int]] = None,
        conta: Optional[Union[str, int]] = None,
        conta_dv: Optional[Union[str, int]] = None,
        nome_empresa: Optional[str] = None,
        mensagem_fixa1: Optional[str] = None,
        mensagem_fixa2: Optional[str] = None,
        numero_remessa: Optional[Union[str, int]] = None,
        **kwargs: dict,
    ):
        set_if_has_value(kwargs, 'codigo_banco', codigo_banco)
        set_if_has_value(kwargs, 'codigo_lote', codigo_lote)
        set_if_has_value(kwargs, 'tipo_registro', tipo_registro)
        set_if_has_value(kwargs, 'operacao', operacao)
        set_if_has_value(kwargs, 'tipo_servico', tipo_servico)
        set_if_has_value(kwargs, 'tipo_inscricao', tipo_inscricao)
        set_if_has_value(kwargs, 'numero_inscricao', numero_inscricao)
        set_if_has_value(kwargs, 'convenio', convenio)
        set_if_has_value(kwargs, 'cobranca', cobranca)
        set_if_has_value(kwargs, 'carteira', carteira)
        set_if_has_value(kwargs, 'variacao', variacao)
        set_if_has_value(kwargs, 'agencia', agencia)
        set_if_has_value(kwargs, 'agencia_dv', agencia_dv)
        set_if_has_value(kwargs, 'conta', conta)
        set_if_has_value(kwargs, 'conta_dv', conta_dv)
        set_if_has_value(kwargs, 'nome_empresa', nome_empresa)
        set_if_has_value(kwargs, 'mensagem_fixa1', mensagem_fixa1)
        set_if_has_value(kwargs, 'mensagem_fixa2', mensagem_fixa2)
        set_if_has_value(kwargs, 'numero_remessa', numero_remessa)
        super().__init__(header, parent, **kwargs)

    def inserir_detalhe(self, **kwargs):
        from .registro3P import BancoBrasil240Registro3P
        return BancoBrasil240Registro3P(self.header, self, self, **kwargs)



from cnab.base.cnab_240 import CNAB240Registro3
from cnab.core.field import CNABField, CNABFieldType

class Santander240Registro3Q(CNAB240Registro3):
    _meta = {
        'codigo_banco': CNABField(
            length=3,
            default='033',
            validation=CNABFieldType.Int,
            required=True
        ),
        'codigo_lote': CNABField(
            length=4,
            default=1,
            validation=CNABFieldType.Int,
            required=True
        ),
        'tipo_registro': CNABField(
            length=1,
            default='3',
            validation=CNABFieldType.Int,
            required=True
        ),
        'numero_registro': CNABField(
            length=5,
            default='0',
            validation=CNABFieldType.Int,
            required=True
        ),
        'seguimento': CNABField(
            length=1,
            default='Q',
            validation=CNABFieldType.Alfa,
            required=True
        ),
        'filler1': CNABField(
            length=1,
            default=' ',
            validation=CNABFieldType.Alfa,
            required=True
        ),
        'codigo_movimento': CNABField(
            length=2,
            default='01',
            validation=CNABFieldType.Int,
            required=True
        ),
        # - ------------------ até aqui é igual para todo registro tipo 3
        'tipo_inscricao': CNABField(
            length=1,
            default='',
            validation=CNABFieldType.Int,
            required=True
        ),
        'numero_inscricao': CNABField(
            length=15,
            default='',
            validation=CNABFieldType.Int,
            required=True
        ),
        'nome_pagador': CNABField(
            length=40,
            default='',
            validation=CNABFieldType.Alfa,
            required=True
        ),
        'endereco_pagador': CNABField(
            length=40,
            default='',
            validation=CNABFieldType.Alfa,
            required=True
        ),
        'bairro_pagador': CNABField(
            length=15,
            default='',
            validation=CNABFieldType.Alfa,
            required=True
        ),
        'cep_pagador': CNABField(
            length=8,
            default='',
            validation=CNABFieldType.Int,
            required=True
        ),
        'cidade_pagador': CNABField(
            length=15,
            default='',
            validation=CNABFieldType.Alfa,
            required=True
        ),
        'uf_pagador': CNABField(
            length=2,
            default='',
            validation=CNABFieldType.Alfa,
            required=True
        ),
        'tipo_incricao_avalista': CNABField(
            length=1,
            default='0',
            validation=CNABFieldType.Int,
            required=True
        ),
        'numero_incricao_avalista': CNABField(
            length=15,
            default='0',
            validation=CNABFieldType.Int,
            required=True
        ),
        'nome_avalista': CNABField(
            length=40,
            default=' ',
            validation=CNABFieldType.Alfa,
            required=True
        ),
        'identificador_carne': CNABField(
            length=3,
            default='0',
            validation=CNABFieldType.Int,
            required=True
        ),
        'sequencia_parcela': CNABField(
            length=3,
            default='0',
            validation=CNABFieldType.Int,
            required=True
        ),
        'total_parcela': CNABField(
            length=3,
            default='0',
            validation=CNABFieldType.Int,
            required=True
        ),
        'numero_plano': CNABField(
            length=3,
            default='0',
            validation=CNABFieldType.Int,
            required=True
        ),
        'filler13': CNABField(
            length=19,
            default=' ',
            validation=CNABFieldType.Alfa,
            required=True
        ),
    }



from typing import Optional
from cnab.base.registro import Registro
from cnab.base.registro_remessa import RegistroRemessa

class CNAB400Registro9(RegistroRemessa):
    def __init__(self, header: Optional[Registro], parent: Optional[Registro], **kwargs: dict):
        super().__init__(header, parent, **kwargs)
        
        self.init_numero_registro()

    def init_numero_registro(self):
        self._data['numero_registro'] = self.header.counter + 1
        self._data['numero_sequencial'] = self.header.counter + 1



from cnab.base.cnab_240 import CNAB240Registro0
from cnab.core.field import (
    CNABField,
    CNABFieldType,
    CNABCreatedDateField,
)
from .registro1 import Santander240Registro1

class Santander240Registro0(CNAB240Registro0):
    registro1_class = Santander240Registro1
    _meta = {
        "codigo_banco": CNABField(
            length=3,
            default="033",
            validation=CNABFieldType.Int,
            required=True,
        ),
        "codigo_lote": CNABField(
            length=4,
            default="0000",
            validation=CNABFieldType.Int,
            required=True,
        ),
        "tipo_registro": CNABField(
            length=1,
            default="0",
            validation=CNABFieldType.Int,
            required=True,
        ),
        "filler1": CNABField(
            length=8,
            default=" ",
            validation=CNABFieldType.Alfa,
            required=True,
        ),
        "tipo_inscricao": CNABField(
            length=1,
            default="",
            validation=CNABFieldType.Int,
            required=True,
        ),
        "numero_inscricao": CNABField(
            length=15,
            default="",
            validation=CNABFieldType.Int,
            required=True,
        ),
        "agencia": CNABField(
            length=4,
            default="",
            validation=CNABFieldType.Int,
            required=True,
        ),
        "filler12": CNABField(
            length=4,
            default="0",
            validation=CNABFieldType.Int,
            required=True,
        ),
        "codigo_beneficiario": CNABField(
            length=6,
            default="0",
            validation=CNABFieldType.Int,
            required=True,
        ),
        "codigo_beneficiario_dv": CNABField(
            length=1,
            default="0",
            validation=CNABFieldType.Int,
            required=True,
        ),
        "filler2": CNABField(
            length=25,
            default=" ",
            validation=CNABFieldType.Alfa,
            required=True,
        ),
        "nome_empresa": CNABField(
            length=30,
            default="",
            validation=CNABFieldType.Alfa,
            required=True,
        ),
        "nome_banco": CNABField(
            length=30,
            default="BANCO SANTANDER",
            validation=CNABFieldType.Alfa,
            required=True,
        ),
        "filler3": CNABField(
            length=10,
            default=" ",
            validation=CNABFieldType.Alfa,
            required=True,
        ),
        "codigo_remessa": CNABField(
            length=1,
            default="1",
            validation=CNABFieldType.Int,
            required=True,
        ),
        "data_geracao": CNABCreatedDateField(
            length=8,
            validation=CNABFieldType.Date,
            required=True,
        ),
        "filler4": CNABField(
            length=6,
            default=" ",
            validation=CNABFieldType.Alfa,
            required=True,
        ),
        "numero_sequencial_arquivo": CNABField(
            length=6,
            default="",
            validation=CNABFieldType.Int,
            required=True,
        ),
        "versao_layout": CNABField(
            length=3,
            default="040",
            validation=CNABFieldType.Int,
            required=True,
        ),
        "filler5": CNABField(
            length=74,
            default=" ",
            validation=CNABFieldType.Alfa,
            required=True,
        ),
    }



from cnab.base.cnab_240 import CNAB240Registro9
from cnab.core.field import CNABField, CNABFieldType


class Caixa240Registro9(CNAB240Registro9):
    _meta = {
        'codigo_banco':CNABField(      #01.5
			length=3,
			default='104',
			validation=CNABFieldType.Int,
			required=True),
		'codigo_lote':CNABField(       #02.5
			length=4,
			default=9999,
			validation=CNABFieldType.Int,
			required=True),
		'tipo_registro':CNABField(     #03.5
			length=1,
			default='9',
			validation=CNABFieldType.Int,
			required=True),
		'filler1':CNABField(          #04.5
			length=9,
			default=' ',
			validation=CNABFieldType.Alfa,
			required=True),
		'qtd_lotes':CNABField(      #05.5
			length=6,
			default='1',
			validation=CNABFieldType.Int,
			required=True),
		'qtd_registros':CNABField(           #06.5
			length=6,
			default='0',
			validation=CNABFieldType.Int,
			required=True),
		'filler2':CNABField(        #07.5
			length=6,
			default=' ',
			validation=CNABFieldType.Alfa,
			required=True),
		'filler3':CNABField(           #08.5
			length=205,
			default=' ',
			validation=CNABFieldType.Alfa,
			required=True),
    }



from typing import Optional, Union
from cnab.base.cnab_400 import CNAB400Registro0
from cnab.core.field import CNABCreatedDateField, CNABField, CNABFieldType
from cnab.utils.dict_utils import set_if_has_value
# from .registro1 import BancoBrasil240Registro1


class ItauCnab400Registro0(CNAB400Registro0):
    _meta = {
        "tipo_registro": CNABField(
            length=1, default="0", validation=CNABFieldType.Int, required=True
        ),
        "operacao": CNABField(
            length=1, default="1", validation=CNABFieldType.Int, required=True
        ),
        "literal_remessa": CNABField(
            length=7, default="remessa", validation=CNABFieldType.Alfa, required=True
        ),
        "tipo_servico": CNABField(
            length=2, default="01", validation=CNABFieldType.Int, required=True
        ),
        "literal_servico": CNABField(
            length=15, default="COBRANCA", validation=CNABFieldType.Alfa, required=True
        ),
        "agencia": CNABField(
            length=4, default="", validation=CNABFieldType.Int, required=True
        ),
        "filler1": CNABField(
            length=2, default="0", validation=CNABFieldType.Int, required=True
        ),
        "conta": CNABField(
            length=5, default="", validation=CNABFieldType.Int, required=True
        ),
        "conta_dv": CNABField(
            length=1, default="", validation=CNABFieldType.Int, required=True
        ),
        "filler2": CNABField(
            length=8, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "nome_empresa": CNABField(
            length=30, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "codigo_banco": CNABField(
            length=3, default="341", validation=CNABFieldType.Int, required=True
        ),
        "nome_banco": CNABField(
            length=15,
            default="BANCO ITAU SA",
            validation=CNABFieldType.Alfa,
            required=True,
        ),
        "data_gravacao": CNABCreatedDateField(
            length=6,
            validation=CNABFieldType.Date,
            required=True,
        ),
        "filler3": CNABField(
            length=294, default=" ", validation=CNABFieldType.Alfa, required=True
        ),
        "numero_sequencial": CNABField(
            length=6, default="1", validation=CNABFieldType.Int, required=True
        ),
    }

    def inserir_detalhe(self, **kwargs):
        from .registro1 import ItauCnab400Registro1
        return ItauCnab400Registro1(self, self, **kwargs)
